import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_screenutil/screen_util.dart';
import 'package:plugin_codelab_new/bloc/feed_cubit.dart';
import 'package:plugin_codelab_new/models/post_comment_model.dart';
import 'package:plugin_codelab_new/models/user_model.dart';
import 'package:plugin_codelab_new/widgets/post.dart';
import 'package:provider/provider.dart';
import '../../providers/messages_provider.dart';
import '../../models/message.dart';
import '../../constants.dart';
import '../../data/socket_io_manager.dart';
import '../widgets/messages_item.dart';
import '../widgets/messages_form.dart';

class ChatScreen extends StatefulWidget {
  final String senderName;
  final String about;
  final String senderImage;
  final FeedPost post;
  final UserModel currentUser;

  const ChatScreen({
    @required this.senderName,
    @required this.about,
    @required this.senderImage,
    @required this.post,
    @required this.currentUser,
  });

  @override
  _ChatScreenState createState() => _ChatScreenState();
}

class _ChatScreenState extends State<ChatScreen> {
  bool isChatBuild;

  List<Message> messagesList;

  Future<UserModel> getUserById(id) async {
    UserModel user = await feedCubit.feedRepository.fetchUserData(id);
    return user;
  }

  ScrollController _scrollController;

  SocketIoManager _socketIoManager;

  bool _isTyping = false;
  String _userNameTyping;

  void _onTyping() {
    _socketIoManager.sendMessage(
        'typing', json.encode({'senderName': widget.senderName}));
  }

  void _onStopTyping() {
    _socketIoManager.sendMessage(
        'stop_typing', json.encode({'senderName': widget.senderName}));
  }

  void _sendMessage(String messageContent) {
    _socketIoManager.sendMessage(
      'send_message',
      Message(
              senderName: widget.senderName,
              about: widget.about,
              senderImage: widget.senderImage,
              content: messageContent,
              date: DateTime.now(),
              postId: this.widget.post.postData.id,
              commentUserId: 1)
          .toJson(),
    );
    Provider.of<MessagesProvider>(context, listen: false).addMessage(
      Message(
          senderName: widget.senderName,
          about: widget.about,
          senderImage: widget.senderImage,
          content: messageContent,
          date: DateTime.now(),
          postId: this.widget.post.postData.id,
          commentUserId: 1),
    );
  }

  FeedCubit feedCubit;
  List<PostCommentModel> commentsList;
  Future<List<PostCommentModel>> getCommentsList() async {
    commentsList = await feedCubit.feedRepository
        .fetchPostComments(this.widget.post.postData.id);
    return commentsList;
  }

  loadData() async {
    await getCommentsList().then(
      (data) {
        setState(
          () {
            commentsList = data;
          },
        );
        _buildChatItems();
      },
    );
  }

  _buildChatItems() async {
    await buildChatItems().then(
      (data) {
        setState(
          () {
            messagesList = data;
          },
        );
        Provider.of<MessagesProvider>(context, listen: false).messages =
            messagesList;
      },
    );
  }

  Future<List<Message>> buildChatItems() async {
    for (PostCommentModel comment in commentsList) {
      Message message;
      await getUserById(comment.user.id).then(
        (user) {
          message = Message(
              senderName: user.firstName + " " + user.lastName,
              about: user.about,
              senderImage: user.profileImg,
              content: comment.comment,
              date: comment.createdAt,
              postId: this.widget.post.postData.id,
              commentUserId: comment.user.id);
        },
      );
      messagesList.add(message);
    }
    return messagesList;
  }

  @override
  void initState() {
    feedCubit = BlocProvider.of<FeedCubit>(context);
    super.initState();
    Provider.of<MessagesProvider>(context, listen: false).messages = [];
    commentsList = [];
    messagesList = [];
    loadData();
    _scrollController = ScrollController();
    _socketIoManager = SocketIoManager(serverUrl: SERVER_URL)
      ..init()
      ..subscribe('receive_message', (Map<String, dynamic> data) {
        if (this.widget.post.postData.id == Message.fromJson(data).postId) {
          Provider.of<MessagesProvider>(context, listen: false)
              .addMessage(Message.fromJson(data));
        }
        _scrollController.animateTo(
          0.0,
          duration: Duration(milliseconds: 200),
          curve: Curves.bounceInOut,
        );
      })
      ..subscribe('typing', (Map<String, dynamic> data) {
        _userNameTyping = data['senderName'];
        setState(() {
          _isTyping = true;
        });
      })
      ..subscribe('stop_typing', (Map<String, dynamic> data) {
        setState(() {
          _isTyping = false;
        });
      })
      ..connect();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    _socketIoManager.disconnect();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
          backgroundColor: Color(0xFF083356),
          title: Padding(
            padding: EdgeInsets.only(
                left: ScreenUtil().setWidth(10), top: ScreenUtil().setWidth(9)),
            child: Text(
              this.widget.post.postData.title,
              style: TextStyle(color: Colors.white),
            ),
          )),
      backgroundColor: Colors.white,
      body: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Expanded(
            child: ListView(
              children: [
                this.widget.post,
                Expanded(
                  child: Consumer<MessagesProvider>(
                    builder: (_, messagesProvider, __) => ListView.builder(
                      shrinkWrap: true,
                      physics: ClampingScrollPhysics(),
                      reverse: true,
                      controller: _scrollController,
                      itemCount: messagesProvider.allMessages.length,
                      itemBuilder: (ctx, index) =>
                          messagesProvider.allMessages[index].postId ==
                                  this.widget.post.postData.id
                              ? MessagesItem(
                                  messagesProvider.allMessages[index],
                                  messagesProvider.allMessages[index]
                                      .isUserMessage(1),
                                )
                              : null,
                    ),
                  ),
                ),
              ],
            ),
          ),
          Visibility(
            visible: _isTyping,
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Row(
                children: <Widget>[
                  Text(
                    '$_userNameTyping is typing',
                    style: Theme.of(context).textTheme.headline6.copyWith(
                          color: Colors.black54,
                          fontWeight: FontWeight.w400,
                          fontSize: 14,
                        ),
                  ),
                ],
              ),
            ),
          ),
          MessageForm(
            postId: this.widget.post.postData.id,
            currentUser: this.widget.currentUser,
            onSendMessage: _sendMessage,
            onTyping: _onTyping,
            onStopTyping: _onStopTyping,
          ),
        ],
      ),
    );
  }
}
